@@ -22,6 +22,12 @@ class GracefulApplyException(Exception):
     pass
 
 
+class ItemSkipped(Exception):
+    """
+    Raised when an item is skipped during `bw verify`.
+    """
+    pass
+
 class NoSuchBundle(Exception):
     """
     Raised when a bundle of unknown name is requested.
@@ -78,20 +84,6 @@ class ItemDependencyError(RepositoryError):
     pass
 
 
-class ItemDependencyLoop(ItemDependencyError):
-    """
-    Raised when there is a loop in item dependencies.
-    """
-    def __init__(self, items):
-        self.items = items
-
-    def __repr__(self):
-        return "<ItemDependencyLoop between {} items>".format(len(self.items))
-
-    def __str__(self):
-        return "<ItemDependencyLoop between {} items>".format(len(self.items))
-
-
 class NoSuchRepository(RepositoryError):
     """
     Raised when trying to get a Repository object from a directory that
