@@ -1,8 +1,8 @@
 from contextlib import suppress
 from datetime import datetime, timedelta
 from hashlib import md5
-from os import environ, mkdir
-from os.path import exists, join
+from os import environ, mkdir, rename
+from os.path import dirname, exists, join
 from threading import Lock
 
 from tomlkit import dumps as toml_dump, parse as toml_parse
@@ -17,22 +17,30 @@ from .exceptions import (
     ItemSkipped,
     NodeLockedException,
     NoSuchBundle,
+    NoSuchGroup,
     RemoteException,
     RepositoryError,
     SkipNode,
 )
-from .group import GROUP_ATTR_DEFAULTS, GROUP_ATTR_TYPES
+from .group import GROUP_ATTR_DEFAULTS, GROUP_ATTR_TYPES, GROUP_ATTR_TYPES_ENFORCED
 from .itemqueue import ItemQueue
 from .items import Item
 from .lock import NodeLock
 from .metadata import hash_metadata
-from .utils import cached_property, error_context, get_file_contents, names
+from .utils import (
+    cached_property,
+    cached_property_set,
+    error_context,
+    get_file_contents,
+    names,
+)
 from .utils.dicts import (
     dict_to_text,
     dict_to_toml,
     diff_dict,
     hash_statedict,
     set_key_at_path,
+    normalize_dict,
     validate_dict,
     COLLECTION_OF_STRINGS,
 )
@@ -56,6 +64,7 @@ from .utils.ui import io
 NODE_ATTR_TYPES = GROUP_ATTR_TYPES.copy()
 NODE_ATTR_TYPES['groups'] = COLLECTION_OF_STRINGS
 NODE_ATTR_TYPES['hostname'] = str
+NODE_ATTRS = sorted(list(GROUP_ATTR_DEFAULTS) + ['bundles', 'file_path', 'groups', 'hostname'])
 
 
 class ApplyResult:
@@ -514,8 +523,11 @@ class Node:
         with error_context(node_name=name):
             validate_dict(attributes, NODE_ATTR_TYPES)
 
+        attributes = normalize_dict(attributes, GROUP_ATTR_TYPES_ENFORCED)
+
         self._add_host_keys = environ.get('BW_ADD_HOST_KEYS', False) == "1"
         self._attributes = attributes
+        self._dynamic_attribute_cache = {}
         self._ssh_conn_established = False
         self._ssh_first_conn_lock = Lock()
         self.file_path = attributes.get('file_path')
@@ -525,13 +537,29 @@ class Node:
         for attr in GROUP_ATTR_DEFAULTS:
             setattr(self, "_{}".format(attr), attributes.get(attr))
 
+    def __getattr__(self, name):
+        with suppress(KeyError):
+            return self._dynamic_attribute_cache[name]
+        try:
+            func = self.repo.node_attribute_functions[name]
+        except KeyError:
+            raise AttributeError(name)
+        else:
+            with error_context(node=self.name, attr=name):
+                value = func(self)
+            self._dynamic_attribute_cache[name] = value
+            return value
+
     def __lt__(self, other):
         return self.name < other.name
 
     def __repr__(self):
         return "<Node '{}'>".format(self.name)
 
-    @cached_property
+    def __str__(self):
+        return self.name
+
+    @cached_property_set
     def bundles(self):
         with io.job(_("{node}  loading bundles").format(node=bold(self.name))):
             bundle_names = set(self._attributes.get('bundles', set()))
@@ -555,7 +583,7 @@ class Node:
     def cdict(self):
         node_dict = {}
         for item in self.items:
-            with suppress(AttributeError):  # actions have no cdict
+            if item.ITEM_TYPE_NAME != 'action':  # actions have no cdict
                 node_dict[item.id] = item.hash()
         return node_dict
 
@@ -577,26 +605,37 @@ class Node:
 
     @property
     def immediate_groups(self):
+        result = set()
+        for group_name in self._attributes.get('groups', set()):
+            try:
+                result.add(self.repo.get_group(group_name))
+            except NoSuchGroup:
+                raise RepositoryError(_(
+                    "Node '{node}' has '{group}' listed as a group, "
+                    "but no such group could be found."
+                ).format(
+                    node=self.name,
+                    group=group_name,
+                ))
         for group in self.repo.groups:
-            if group.name in self._attributes.get('groups', set()):
-                yield self.repo.get_group(group.name)
+            if group in result:
+                continue
             elif self in group._nodes_from_members:
-                yield self.repo.get_group(group.name)
+                result.add(self.repo.get_group(group.name))
             else:
                 for pattern in group._member_patterns:
                     if pattern.search(self.name) is not None:
-                        yield self.repo.get_group(group.name)
+                        result.add(self.repo.get_group(group.name))
+        return result
 
-    @cached_property
+    @cached_property_set
     @io.job_wrapper(_("{}  determining groups").format(bold("{0.name}")))
     def groups(self):
-        result = set()
         for group in self.immediate_groups:
-            result.add(group)
+            yield group
             for parent_group in group.parent_groups:
                 # these are not just the *immediate* parent groups
-                result.add(parent_group)
-        return result
+                yield parent_group
 
     def has_any_bundle(self, bundle_list):
         for bundle_name in bundle_list:
@@ -625,7 +664,7 @@ class Node:
                 return True
         return False
 
-    @cached_property
+    @cached_property_set
     def items(self):
         items = {}
         if not self.dummy:
@@ -641,12 +680,30 @@ class Node:
                         ))
                     else:
                         items[item.id] = item
-        return set(items.values())
+        return items.values()
 
     @cached_property
     def magic_number(self):
         return int(md5(self.name.encode('UTF-8')).hexdigest(), 16)
 
+    def check_connection(self):
+        try:
+            # Running "true" is meant to catch connection errors early,
+            # but this only works on UNIX-y systems (i.e., not k8s).
+            if self.os in self.OS_FAMILY_UNIX:
+                self.run("true")
+            elif self.os == 'routeros':
+                self.run_routeros("/nothing")
+        except RemoteException as exc:
+            io.stdout(_("{x} {node}  Connection error: {msg}").format(
+                msg=exc,
+                node=bold(self.name),
+                x=red("!"),
+            ))
+            return False
+        else:
+            return True
+
     def apply(
         self,
         autoskip_selector=(),
@@ -700,19 +757,8 @@ class Node:
             x=blue("i"),
         ))
 
-        error = False
-
-        try:
-            # Running "true" is meant to catch connection errors early,
-            # but this only works on UNIX-y systems (i.e., not k8s).
-            if self.os in self.OS_FAMILY_UNIX:
-                self.run("true")
-        except RemoteException as exc:
-            io.stdout(_("{x} {node}  Connection error: {msg}").format(
-                msg=exc,
-                node=bold(self.name),
-                x=red("!"),
-            ))
+        error = None
+        if not self.check_connection():
             error = _("Connection error (details above)")
             item_results = []
         else:
@@ -820,6 +866,16 @@ class Node:
         """
         return self.metadata
 
+    def rename(self, new_name):
+        if not self.is_toml:
+            raise ValueError(_(
+                "{node} cannot be renamed: not in TOML format"
+            ).format(node=self.name))
+        new_path = join(dirname(self.file_path), f"{new_name}.toml")
+        rename(self.file_path, new_path)
+        self.file_path = new_path
+        self.name = new_name
+
     def run(self, command, data_stdin=None, may_fail=False, log_output=False, user="root"):
         assert self.os in self.OS_FAMILY_UNIX
 
@@ -843,7 +899,12 @@ class Node:
             if self._ssh_first_conn_lock.acquire(False):
                 try:
                     with io.job(_("{}  establishing connection...").format(bold(self.name))):
-                        operations.run(self.hostname, "true", add_host_keys=self._add_host_keys)
+                        operations.run(
+                            self.hostname,
+                            "true",
+                            add_host_keys=self._add_host_keys,
+                            username=self.username,
+                        )
                     self._ssh_conn_established = True
                 finally:
                     self._ssh_first_conn_lock.release()
@@ -866,9 +927,22 @@ class Node:
             user=user,
         )
 
+    def run_routeros(self, *command):
+        assert self.os == 'routeros'
+        return operations.run_routeros(
+            self.hostname,
+            self.username,
+            self.password,
+            *command,
+        )
+
+    @property
+    def is_toml(self):
+        return self.file_path and self.file_path.endswith(".toml")
+
     @cached_property
     def toml(self):
-        if not self.file_path or not self.file_path.endswith(".toml"):
+        if not self.is_toml:
             raise ValueError(_("node {} not in TOML format").format(self.name))
         return toml_parse(get_file_contents(self.file_path))
 
@@ -919,6 +993,8 @@ class Node:
 
         if not self.items:
             io.stdout(_("{x} {node}  has no items").format(node=bold(self.name), x=yellow("!")))
+        elif not self.check_connection():
+            io.stdout(_("{x} {node}  Connection error (details above)").format(node=bold(self.name), x=red("!")))
         else:
             result = verify_items(
                 self,
