@@ -5,7 +5,10 @@ from os import listdir, mkdir, walk
 from os.path import abspath, dirname, isdir, isfile, join
 
 from pkg_resources import DistributionNotFound, require, VersionConflict
-from tomlkit import parse as toml_parse
+try:
+    from tomllib import loads as toml_load
+except ImportError:
+    from rtoml import load as toml_load
 
 from . import items, VERSION_STRING
 from .bundle import FILENAME_ITEMS
@@ -18,7 +21,7 @@ from .exceptions import (
 )
 from .group import Group
 from .metagen import MetadataGenerator
-from .node import Node
+from .node import Node, NODE_ATTRS
 from .secrets import FILENAME_SECRETS, generate_initial_secrets_cfg, SecretProxy
 from .utils import (
     cached_property,
@@ -27,7 +30,7 @@ from .utils import (
     names,
 )
 from .utils.scm import get_git_branch, get_git_clean, get_rev
-from .utils.dicts import hash_statedict, untoml
+from .utils.dicts import hash_statedict
 from .utils.text import mark_for_translation as _, red, validate_name
 from .utils.ui import io
 
@@ -192,6 +195,7 @@ class Repository(MetadataGenerator):
         self.bundle_names = []
         self.group_dict = {}
         self.node_dict = {}
+        self.node_attribute_functions = {}
         self._get_all_attr_code_cache = {}
         self._get_all_attr_result_cache = {}
 
@@ -332,12 +336,21 @@ class Repository(MetadataGenerator):
         return env
 
     def nodes_or_groups_from_file(self, path, attribute, preexisting):
+        def node_attribute(func):
+            if func.__name__ in NODE_ATTRS:
+                raise RepositoryError(_(
+                    "cannot redefine builtin attribute '{attr}' as @node_attribute in nodes.py"
+                ).format(attr=func.__name__))
+            self.node_attribute_functions[func.__name__] = func
+            return func
+
         try:
             flat_dict = self.get_all_attrs_from_file(
                 path,
                 base_env={
                     attribute: preexisting,
                     'libs': self.libs,
+                    'node_attribute': node_attribute,
                     'repo_path': self.path,
                     'vault': self.vault,
                 },
@@ -359,6 +372,7 @@ class Repository(MetadataGenerator):
         path = join(self.path, directory)
         if not isdir(path):
             return
+        result = {}
         for root_dir, _dirs, files in walk(path):
             for filename in files:
                 filepath = join(root_dir, filename)
@@ -366,10 +380,20 @@ class Repository(MetadataGenerator):
                         not isfile(filepath) or \
                         filename.startswith("_"):
                     continue
+                entity_name = filename[:-5]
+                if entity_name in result:
+                    raise RepositoryError(_(
+                        "Duplicate definition of {entity_name} in {file1} and {file2}"
+                    ).format(
+                        entity_name=entity_name,
+                        file1=filepath,
+                        file2=result[entity_name]['file_path'],
+                    ))
                 with error_context(filepath=filepath):
-                    infodict = untoml(toml_parse(get_file_contents(filepath)))
+                    infodict = toml_load(get_file_contents(filepath).decode())
                 infodict['file_path'] = filepath
-                yield filename[:-5], infodict
+                result[entity_name] = infodict
+        return result
 
     def items_from_dir(self, path):
         """
@@ -519,7 +543,7 @@ class Repository(MetadataGenerator):
                 self.bundle_names.append(dir_entry)
 
         # populate groups
-        toml_groups = dict(self.nodes_or_groups_from_dir("groups"))
+        toml_groups = self.nodes_or_groups_from_dir("groups")
         self.group_dict = {}
         for group in self.nodes_or_groups_from_file(self.groups_file, 'groups', toml_groups):
             self.add_group(Group(*group))
@@ -530,7 +554,7 @@ class Repository(MetadataGenerator):
             self.item_classes.append(item_class)
 
         # populate nodes
-        toml_nodes = dict(self.nodes_or_groups_from_dir("nodes"))
+        toml_nodes = self.nodes_or_groups_from_dir("nodes")
         self.node_dict = {}
         for node in self.nodes_or_groups_from_file(self.nodes_file, 'nodes', toml_nodes):
             self.add_node(Node(*node))
