@@ -1,5 +1,5 @@
 from base64 import b64encode, urlsafe_b64decode
-from configparser import ConfigParser
+from configparser import ConfigParser, NoOptionError
 import hashlib
 import hmac
 from os import environ
@@ -101,7 +101,7 @@ class SecretProxy:
         returns the plaintext as base64.
         """
         if environ.get("BW_VAULT_DUMMY_MODE", "0") != "0":
-            return b64encode("decrypted file as base64").decode('utf-8')
+            return b64encode(b"decrypted file as base64").decode('utf-8')
 
         cryptotext = get_file_contents(join(self.repo.data_dir, source_path))
         key, cryptotext = self._determine_key_to_use(cryptotext, key, source_path)
@@ -246,7 +246,16 @@ class SecretProxy:
             return {}
         result = {}
         for section in config.sections():
-            result[section] = config.get(section, 'key').encode('utf-8')
+            try:
+                result[section] = config.get(section, 'key').encode('utf-8')
+            except NoOptionError:
+                result[section] = run(
+                    config.get(section, 'key_command'),
+                    check=True,
+                    shell=True,
+                    stdout=PIPE,  # replace with capture_output=True
+                                  # when dropping support for Python 3.6
+                ).stdout.strip()
         return result
 
     @staticmethod
