@@ -22,7 +22,7 @@ STDERR_WRITER = getwriter('utf-8')(stderr.buffer)
 STDOUT_WRITER = getwriter('utf-8')(stdout.buffer)
 
 
-def cached_property(prop):
+def cached_property(prop, convert_to=None):
     """
     A replacement for the property decorator that will only compute the
     attribute's value on the first call and serve a cached copy from
@@ -38,12 +38,27 @@ def cached_property(prop):
                     return_value = tuple(return_value)
             except DontCache as exc:
                 return exc.obj
+            except AttributeError as exc:
+                # It's bad to raise an AttributeError from a property
+                # since this will cause Python to try and use
+                # __getattr__, thinking the property doesn't exist. The
+                # original exception gets swallowed by this process,
+                # which leads to entriely unhelpful tracebacks.
+                # To prevent this, we wrap AttributeErrors in a generic
+                # Exception.
+                raise Exception("AttributeError in property, see above") from exc
             else:
+                if convert_to:
+                    return_value = convert_to(return_value)
                 self._cache[prop.__name__] = return_value
         return self._cache[prop.__name__]
     return property(cache_wrapper)
 
 
+def cached_property_set(prop):
+    return cached_property(prop, convert_to=set)
+
+
 def download(url, path):
     with error_context(url=url, path=path):
         if not exists(dirname(path)):
@@ -71,31 +86,11 @@ def error_context(**kwargs):
     we're processing lots of different dicts, a "KeyError: foo" will
     often not be helpful, since it's not clear which dict is missing the
     key.
-
-
-    >>> with error_context(arbitrary_kwarg="helpful hint"):
-    ...     {}["foo"]
-    ...
-    Traceback (most recent call last):
-      [...]
-    KeyError: 'foo'
-
-    The above exception was the direct cause of the following exception:
-
-    Traceback (most recent call last):
-      [...]
-    bundlewrap.utils.ErrorContext: ACTUAL EXCEPTION ABOVE
-    {'arbitrary_kwarg': 'helpful hint'}
-
-
-    Careful though: Only use this in places where you don't expect
-    exceptions to occur, since they will indiscriminately be reraised as
-    ErrorContext.
     """
     try:
         yield
     except Exception as exc:
-        raise ErrorContext("ACTUAL EXCEPTION ABOVE\n" + repr(kwargs)) from exc
+        raise exc from ErrorContext(repr(kwargs))
 
 
 class Fault:
