@@ -1,3 +1,5 @@
+from os.path import join
+
 from bundlewrap.utils.testing import make_repo, run
 
 
@@ -76,3 +78,40 @@ def test_bundles_via_group(tmpdir):
     assert stdout.decode().strip().split("\n") == ["bundle1", "bundle2", "bundle3"]
     assert stderr == b""
     assert rcode == 0
+
+
+def test_all_attrs(tmpdir):
+    make_repo(
+        tmpdir,
+        nodes={
+            "node1": {
+                'username': "potato",
+            },
+        },
+    )
+    stdout, stderr, rcode = run("bw nodes -a all -- node1", path=str(tmpdir))
+    assert "potato" in stdout.decode()
+    assert stderr == b""
+    assert rcode == 0
+
+
+def test_dynamic_attrs(tmpdir):
+    make_repo(
+        tmpdir,
+        groups={"all": {'member_patterns': {r".*"}}},
+        nodes={
+            "node1": {},
+            "node2": {},
+            "node3": {},
+        },
+    )
+    with open(join(tmpdir, "nodes.py"), 'a') as f:
+        f.write("""
+@node_attribute
+def dynamic(node):
+    return node.name + "DYNAMIC"
+""")
+    stdout, stderr, rcode = run("BW_TABLE_STYLE=grep bw nodes all -a dynamic | cut -f 2", path=str(tmpdir))
+    assert stdout.decode().strip().split("\n") == ["node1DYNAMIC", "node2DYNAMIC", "node3DYNAMIC"]
+    assert stderr == b""
+    assert rcode == 0
